#!/bin/sh
# review/fix file permissions and size for runtime-test suite
# Stew Benedict, FSG, Dec 21 2006

show_log() {
    if [ -z $QUIET ];then
        echo -e "\n$@"
    fi
}

process_entry() {
    if [ "$3" != "$uid" ]; then
        show_log "$path \n\tuid wrong: $3, should be $uid, fixing..."
	if [ -z "$DRYRUN" ]; then
	    chown $uid $path
	fi
        DIRTY=1
    fi
    if [ "$4" != "$gid" ]; then
        show_log "$path \n\tgid wrong: $4, should be $gid, fixing..."
	if [ -z "$DRYRUN" ]; then
            chgrp $gid $path
	fi
        DIRTY=1
    fi
    if [ "$1" != "$attr" ]; then
        setuidb=`echo $attr | sed 's|**s|1|g' | cut -c4,7,10 | sed 's|x|0|g;s|-|0|g'`
        setuido=`echo "obase=8; ibase=2; $setuidb" | bc`
	
        numattr=`echo $attr | sed  's|**s|x|g;s|rwx|7|g;s|rw-|6|g;s|r-x|5|g;s|r--|4|g;s|-wx|3|g;s|-w-|2|g;s|--x|1|g;s|---|0|g' | cut -c -4 | cut -c 2-`
        show_log "$path \n\tattr wrong: $1, should be $attr($setuido$numattr), fixing..."
	if [ -z "$DRYRUN" ]; then
            chmod $setuido$numattr $path
	fi
        DIRTY=1
    fi
    if [ "$5" != "$size" -a -n "$SIZE" ]; then
        show_log "$path \n\tsize wrong: $5, should be $size, can't fix this..."
        DIRTY=1
    fi
}

DIRTY=0

for args in $@; do
    if [ "$args" = "-q" ];then 
        QUIET=1
    fi
    if [ "$args" = "-s" ];then 
        SIZE=1
    fi
    if [ "$args" = "-d" ];then 
        DRYRUN=1
    fi
    if [ "$args" = "-p" ];then 
        NOPROGRESS=1
    fi
    if [ "$args" = "-?" -o "$args" = "-h" ];then
	echo "Usage: $0 -q (quiet) -s (check sizes) -d (dry-run) -p (no progress)"
	exit 0
    fi	
done

while read attr foo uid gid size month day year path; do
    INSTALLED=`ls -ld $path 2>/dev/null`
    if [ -n "$INSTALLED" ]; then
        process_entry $INSTALLED
	if [ -z "$NOPROGRESS" ];then
	    echo -n "."
	fi
    else
	show_log "File: $path \n\tis not installed - Please reinstall runtime test."
    fi
done < /opt/lsb/doc/lsb-test-core/runtime-files.txt

if [ "$DIRTY" -eq 0 ];then
    echo -e "\nNo problems found..."
else
    echo -e "\nProblems found, attempted repair..."
fi
