: use Bourne Shell
#########################################################################
#									#
#	SCCS:	@(#)common/vprog/vmake.sh	4.2 (02/03/97)  VSXgen release 1.4#
#									#
#	vmake.sh: execute make command			 		#
#									#
#	Usage:  vmake.sh target						#
#									#
#	vmake.sh -	called by vbuild to execute make command(s)	#
#									#
#			Make specified target.	If target is INSTALL,	#
#			determine whether macro definition of test	#
#			function exists.  If so install separate	#
#			macro testset(s) in M<testset> directory.	#
#			If not, create a dummy macro testset which	#
#			gives NOTINUSE results.				#
#									#
#	AUTHOR:		Geoff Clare, UniSoft Ltd.			#
#	DATE CREATED:	30 Sept 1988				  	#
#	MODIFIED:							#
#									#
#		(earlier mods history deleted)				#
#									#
#		Geoff Clare, 2 Oct 1991					#
#			Install macro version directly in M directory	#
#			by passing INSTDIR down to make, as mv may	#
#			remove privileges.				#
#									#
#		Geoff Clare, 6 Nov 1991					#
#			Changes due to new directory structure		#
#									#
#		Geoff Clare, 10 Mar 1992				#
#			Implement new dummy macro testsets which	#
#			don't use the TET shell API.			#
#									#
#		Geoff Clare, 22 Sept 1992				#
#			Changes for testing XNFS			#
#									#
#		Geoff Clare, 20 Dec 1993				#
#			Changes for DTET2				#
#									#
#		Geoff Clare, October 1996				#
#			Added TETware support				#
#									#
#		Geoff Clare, Jan-Feb 1997				#
#			Changes for VSXgen				#
#									#
#########################################################################

if test $# -ne 1
then
	echo >&2 "usage: $0 target"
	exit 1
fi

target="$1"

# Obtain current parameter settings

VSXDIR=${VSXDIR:-$HOME/SRC}
. $VSXDIR/vsxparams

VSX_MAKE=${VSX_MAKE:-make}
TET_ROOT=${TET_ROOT:-$VSXDIR/../..}
TET_EXECUTE=${TET_EXECUTE:-$VSXDIR/../TESTROOT}
CC=${CC:-cc}

# Decide which TET directories to use - prefer tet3 over dtet2/posix_c
case $TEST_XNFS in
y|Y)
	if test -d $TET_ROOT/inc/tet3 -a -d $TET_ROOT/lib/tet3
	then tetdir=tet3
	elif test -d $TET_ROOT/inc/dtet2 -a -d $TET_ROOT/lib/dtet2
	then tetdir=dtet2
	else
		echo >&2 "$0: could not find tet3 or dtet2 inc/lib directories"
		exit 1
	fi

	XNFSLIB=$VSXDIR/LIB/xnfslib.a
	;;
*)
	if test -d $TET_ROOT/inc/tet3 -a -d $TET_ROOT/lib/tet3
	then tetdir=tet3
	elif test -d $TET_ROOT/inc/posix_c -a -d $TET_ROOT/lib/posix_c
	then tetdir=posix_c
	else
		echo >&2 "$0: could not find tet3 or posix_c inc/lib directories"
		exit 1
	fi

	XNFSLIB=
	;;
esac
TETINC=$VSXDIR/../../inc/$tetdir
APILIB=$VSXDIR/../../lib/$tetdir/libapi.a
TCM=$VSXDIR/../../lib/$tetdir/tcm.o
TCMCHILD=$VSXDIR/../../lib/$tetdir/tcmchild.o
THRAPILIB=$VSXDIR/../../lib/$tetdir/libthrapi.a
THRTCM=$VSXDIR/../../lib/$tetdir/thrtcm.o
THRTCMCHILD=$VSXDIR/../../lib/$tetdir/thrtcmchild.o

# Make command with parameters set on command line (use: eval $make_cmd target)
make_cmd='$VSX_MAKE VSXDIR="$VSXDIR" TET_EXECUTE="$TET_EXECUTE"
	TETINC="$TETINC" APILIB="$APILIB" TCM="$TCM" TCMCHILD="$TCMCHILD"
	THRAPILIB="$THRAPILIB" THRTCM="$THRTCM" THRTCMCHILD="$THRTCMCHILD"
	INCDIRS="$INCDIRS" CC="$CC" COPTS="$COPTS" THR_COPTS="$THR_COPTS"
	DEFINES="$DEFINES" LDFLAGS="$LDFLAGS" AR="$AR"
	LORDER="$LORDER" TSORT="$TSORT" RANLIB="$RANLIB" MLIB="$MLIB"
	RPCLIB="$RPCLIB" XNFSLIB="$XNFSLIB" SYSLIBS="$SYSLIBS"'

for subset in $SUBSETS
do
	for var in `sed -n 's/^\([^#][^=]*\)=.*/\1/p' $VSXDIR/subsets/$subset/config_params`
	do
		eval "make_cmd=\"\$make_cmd\"' \"$var=\$$var\"'"
	done
done

ctmp=vmake_tmp.c
etmp=vmake_tmp.E

# Some shells ignore "/" in IFS, hence do this with "tr":
set -- `pwd |tr / ' '`
while test $# -gt 3
do shift
done

sect="$1"
area="$2"
tset="$3"
tdir="$TET_EXECUTE/tset/$sect/$area/$tset"
Mtdir="$TET_EXECUTE/tset/$sect/$area/M$tset"

if test ! -d "$tdir"
then
	echo >&2 "$0: install directory \"$tdir\" does not exist"
	exit 1
fi

# For XNFS testing none of the remaining macro-related stuff is needed
# so just execute "make" and exit.

case $TEST_XNFS in
y|Y)
	eval $make_cmd '"$target"' || exit 1
	exit 0
	;;
esac

# If the file tset_list exists, read a list of target testsets from it
# (e.g. printf is also installed as fprintf and sprintf).

if test -f tset_list
then
	testsets=`cat tset_list`
	if test -z "$testsets"
	then
		echo >&2 "$0: bad tset_list file in `pwd`"
		exit 1
	fi
else
	testsets="$tset"
fi

# If target is INSTALL do a make CLEAN first to safeguard against
# using a "functions_only" .o file left over from a previous build.
# These can only exist if a ".macros_exist" file has been created.

if test X"$target" = X"INSTALL" -a -f .macros_exist
then
	eval $make_cmd CLEAN || exit 1
fi

# Clobber any macro version executables if necessary

if test X"$target" = X"CLOBBER" -a -d "$Mtdir"
then
	echo "	rm -f" $Mtdir/*
	rm -f $Mtdir/*
fi

# Execute the make command for function versions

SAVDEF=$DEFINES
DEFINES="$DEFINES -DUNDEF_MACROS"
eval $make_cmd '"$target"' || exit 1
DEFINES=$SAVDEF

# If target was not INSTALL, no more to do

if test X"$target" != X"INSTALL"
then
	exit 0
fi

# Check for macros for the functions being tested

# Only required for '.os' sections
case $sect in
*.os) ;;
*) exit 0 ;;
esac

# Make the macro directory

if test ! -d "$Mtdir"
then
	mkdir "$Mtdir" || exit 1
fi

# Get list of functions tested by this testset

functions=`sed -n '/^#[	 ]*ifdef[	 ]*UNDEF_MACROS/,/^#[	 ]*endif/ s/^#[	 ]*undef//p' $tset.c`
case $functions in
"")
	echo >&2 "$0: missing or bad UNDEF_MACROS block in `pwd`/$tset.c"
	exit 1
	;;

UNDEF_MACROS)
	# allowed to be only a macro (e.g. assert)
	macros=n
	;;

*)
	# See if a separate macro version is required

	trap "rm -f $ctmp $etmp; exit 1" 1 2 3 15

	cat "$tset".c >$ctmp
	for i in $functions
	do
		cat << EOF >>$ctmp
#ifdef $i
MACRO_EXISTS("$i");
#endif
EOF
	done

	$CC -I. -I"$TETINC" -I"$VSXDIR"/INC -I"$VSXDIR"/SYSINC \
		$COPTS $DEFINES -E $ctmp > $etmp || exit 1

	macros=n
	for i in $functions
	do
		if grep "MACRO_EXISTS(\"$i\");" $etmp >/dev/null
		then
			macros=y
		fi
	done

	rm -f $ctmp $etmp
	trap 1 2 3 15
	;;
esac

if test "$macros" = n
then
	# Create a dummy macro testset

	# Extract number of tests from the line
	#     #define NO_TESTS  <N>
	# in the testset source file.
	NO_TESTS=`sed -n 's/^#[ 	]*define[ 	]*NO_TESTS[ 	]*\([0-9][0-9]*\).*$/\1/p' "$tset".c`
	if test 0 -ge "$NO_TESTS"
	then
		echo >&2 "$0: bad NO_TESTS definition in \"$tset.c\""
		exit 1
	fi

	for i in $testsets
	do
		awk 'END {
		    print "TET_ACTIVITY=${TET_ACTIVITY:-0}"
		    print "time=`date +\"%H:%M:%S\"`"
		    print "cat >tet_xres <<!"
		    print "15|$TET_ACTIVITY dummy 1|TCM Start"
		    printf "400|$TET_ACTIVITY 1 %d $time|IC Start\n", NO_TESTS
		    for (i = 1; i <= 0 + NO_TESTS; i++)
		    {
			printf "200|$TET_ACTIVITY %d $time|TP Start\n", i
			printf "520|$TET_ACTIVITY %d $$ 1 1|", i
			print "No macros defined or no macro tests required"
			printf "220|$TET_ACTIVITY %d 3 $time|NOTINUSE\n", i
		    }
		    printf "410|$TET_ACTIVITY 1 %d $time|IC End\n", NO_TESTS
		    print "!"
		}' NO_TESTS="$NO_TESTS" /dev/null > "$Mtdir"/T.$i &&
		chmod a+x "$Mtdir"/T.$i || exit 1
	done

	exit 0
fi
	
# Install macro version of testset in $Mtdir

echo "
Macros exist for this testset - installing separate macro version
"

# Clean out the existing .o's or they will get reused

eval $make_cmd CLEAN || exit 1

# Place marker to warn later builds to clean out ".o"s
> .macros_exist

# Execute the make command for macro versions

eval $make_cmd 'INSTDIR="$Mtdir"' INSTALL || exit 1
