#########################################################################
#									#
#	SCCS:	@(#)common/vrpt/vrptsum.awk	4.2 (07/01/97)  VSXgen release 1.4#
#									#
#	UniSoft Ltd., London, England
#									#
# (C) Copyright 1991 X/Open Company Limited
#
# All rights reserved.  No part of this source code may be reproduced,	#
# stored in a retrieval system, or transmitted, in any form or by any	#
# means, electronic, mechanical, photocopying, recording or otherwise,	#
# except as stated in the end-user licence agreement, without the prior	#
# permission of the copyright owners.					#
#									#
# X/Open and the 'X' symbol are trademarks of X/Open Company Limited in	#
# the UK and other countries.						#
#									#
#########################################################################

#########################################################################
#									#
# NAME:		vrptsum.awk - produce summary report from		#
#				vtest/vmake output			#
#									#
# PROJECT:	VSX (X/OPEN Validation Suite)				#
#									#
# AUTHOR:	Geoff Clare, UniSoft Ltd.				#
#									#
# DATE:		1st June 1988						#
#									#
# MODIFICATIONS:							#
#	(earlier mods history deleted).					#
#									#
#	Geoff Clare, 20 Oct 1992					#
#		Changes for XNFS testing				#
#									#
#	Geoff Clare, 16 Jan 1997					#
#		Changes for VSXgen					#
#									#
#	Geoff Clare, July 1997						#
#		Avoid use of printf "%s" with v. large strings		#
#									#
#########################################################################

BEGIN {
	sectid = osection = ""
	TEST_MODE="(unknown)"
	XNFS="n"
	EXP="exp"
	prog = "Test"
	months = "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec "
	sectnum = 0
	packagenum = 0
	vtermok = "YES"
	incomplete = "NO"
	areatype = "F"
	FS = "|"
}

NR == 1 { DTE = $0; next }
NR == 2 { CMD = $0; next }
NR == 3 { LEVEL = $0; next }
NR == 4 { RANGE = $0; next }
NR == 5 { DEBUG = $0; next }
NR == 6 { HEADER = $0; next }
NR == 7 { COVER = $0; next }
NR == 8 { LINES = $0; next }
NR == 9 { COLS = 0 + $0; next }
NR == 10 { NAMES = $0; next }
NR == 11 { VERBOSE = $0; next }
NR == 12 { TRUNCATE = $0; next }
NR == 13 { PAGENUMB = $0; next }

# The most frequent pattern matches are at the top for speed

prog == "Make" { next }  # Don't want a summary for make/clean reports
prog == "Clean" { next } # (if just exit vrpt can't write to pipe)

$1 == "520" {
	# Test Information
	# Since the result code is after this info in the journal,
	# we have to store the info for printing after the result.
	# Some awk's can't handle the amount of data to be stored,
	# so truncate after the number of lines specified in TRUNCATE.
	# (TRUNCATE == 0 means no truncation).

	sav_ti = ""
	sav_cnt = 0
	server_ti = ""
	server_cnt = 0
	ok = 1
	while ($1 == "520" && ok > 0)
	{
	    if ($3 != "")
	    {
		split($2, tmp, " ")
		# DTET context numbers start with 3 digit system number
		if (tmp[3] ~ /^001/)
		{
		    if (TRUNCATE+0 > 0 && server_cnt == TRUNCATE+0)
			text_part = "TEST INFORMATION TRUNCATED BY -t OPTION"
		    else
		    {
			text_part = $3
			for (i = 4; i <= NF+0; i++)
			    text_part = text_part "|" $i
		    }
		    if (TRUNCATE+0 <= 0 || server_cnt++ <= TRUNCATE)
			server_ti = server_ti "\t" text_part "\n"
		}
		else
		{
		    if (TRUNCATE+0 > 0 && sav_cnt == TRUNCATE+0)
			text_part = "TEST INFORMATION TRUNCATED BY -t OPTION"
		    else
		    {
			text_part = $3
			for (i = 4; i <= NF+0; i++)
			    text_part = text_part "|" $i
		    }
		    if (TRUNCATE+0 <= 0 || sav_cnt++ <= TRUNCATE)
			sav_ti = sav_ti "\t" text_part "\n"
		}
	    }
	    ok = getline
	}
	# no "next" here: must drop through after getline
}

$1 == "-3" && fip_cnt > 0 && fip_td[fip_cnt] == "" {
	# Test Description

	ok = 1
	while ($1 == "-3" && ok > 0)
	{
	    if ($3 != "")
	    {
		fip_td[fip_cnt] = fip_td[fip_cnt] "\t" $3
		for (i = 4; i <= NF+0; i++)
		    fip_td[fip_cnt] = fip_td[fip_cnt] "|" $i
		fip_td[fip_cnt] = fip_td[fip_cnt] "\n"
	    }
	    ok = getline
	}
	# no "next" here: must drop through after getline
}

$1 == "220" {
	# TP Result

	split($2, tpres, " ")
	if (tpres[3] != "7")	# don't include NORESULT
		count[sectid "," areatype ",exe"]++
	if (tpres[3] == "0")	# report a pass
		count[sectid "," areatype ",suc"]++
	else
		count[sectid "," areatype "," tpres[3]]++
	if (tpres[3] == "102")
	{
		fip_cnt++
		fip_id[fip_cnt] = tset_name " " tpres[2]
		fip_td[fip_cnt] = ""
		if (server_ti == "")
			fip_ti[fip_cnt] = sav_ti
		else
			fip_ti[fip_cnt] = sav_ti \
			    "\n\t\tTest Information From Server:\n" server_ti
	}
	next
}

$1 == "0" {
	# TCC Start

	# Extract test date
	n = split($2, v, " ")
	if (n >= 3)
	{
	    # New format (4.1.2 on): 3rd parameter is YYYYMMDD
	    i = substr(v[3], 5, 2) # month number
	    month = substr(months, i*4-3, 4)
	    vf_date = month substr(v[3], 7, 2) ", " substr(v[3], 1, 4)
	    vh_date = vf_date " " v[2]
	}
	else
	{
	    # Old 4.1.1 format
	    # This relies on undocumented format of text part of TCC Start line
	    n = split($3, v, " ")
	    vh_date = v[8] " " v[9] " " v[10] " " v[11] " " v[12]
	    vf_date = v[9] " " v[10] ", " v[12]
	}

	next
}

$1 == "20" {
	# Config Start

	# Determine if build or clean mode
	if ($2 ~ /0 *$/)
	{
		prog = "Make"
	}
	else if ($2 ~ /2 *$/)
	{
		prog = "Clean"
	}
}

$1 == "30" {
	# Config Parameter

	n = split($3, tmp, "=")
	vp[vpcnt",1"] = tmp[1]
	if (vp[vpcnt",1"] == "VSX_ORG")
	    vsx_org = tmp[2]
	else if (vp[vpcnt",1"] == "VSX_SYS")
	    vsx_sys = tmp[2]
	else if (vp[vpcnt",1"] == "TEST_MODE")
	    TEST_MODE = tmp[2]
	else if (vp[vpcnt",1"] == "TEST_PACKAGES")
	    packagenum = split(tmp[2], packages, " ")
	else if (vp[vpcnt",1"] == "TEST_XNFS")
	{
	    if (tmp[2] ~ /^[yY]/)
		XNFS = "y"
	}
	vp[vpcnt++",2"] = tmp[2]
	next
}

$1 == "-1" {
	# Start new journal file

	jnlfilenum++
	jnlfiles[jnlfilenum] = $2
	if (vtermok == "NO")
		incomplete = "YES"
	vtermok = "NO"
	next
}

$1 == "10" || $1 == "900" {
        # TC Start or TCC End

	if ($1 == "900")
	{
		vtermok = "YES"
		sectid = osection
	}
	else
	{
		split($2, tcs, " ")
		tset_name = tcs[2]
		n = split(tset_name, comp, "/")
		m = split(comp[n], tc, ".")
		testset = tc[m]
		sectid = comp[n-3]
	}

	if (comp[n-1] ~ /M/)
		areatype = "M"
	else
		areatype = "F"

	if (sectid != osection || $1 == "900")
	{
		if (sectid != osection)
		{
			sectnum++
			sections[sectnum] = sectid
		}
		osection = sectid
	}
	next
}

$1 == "70" {
	# Scenario Information Line (contains expected counts)

	n = split($3, scen_info, " ")
	split(scen_info[n], exp_count, "\"")
	sector = scen_info[n-1]
	count[sector ",F," EXP] = exp_count[1]
	if (XNFS == "n" && sector ~ /\.os$/)
		count[sector ",M," EXP] = exp_count[1]
	next
}

END {
	if (prog == "Make" || prog == "Clean")
		exit	# no summary for make/clean reports


	# Parameters for vrpt.page
	printf "%s\n%s\n%s\n%s\n", HEADER, LEVEL, LINES, COLS
	printf "Test Agency: %s\n", vsx_org
	printf "System Tested: %s\n", vsx_sys
	printf "Test Date:   %s\n", vf_date
	printf "%s\n", PAGENUMB

	printf "\n\n\n\n\n"
	str = "X/OPEN Verification Suite " LEVEL " Summary Report"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n\n\n"
	str = "VSXgen 1.4"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	for (i = 1; i <= packagenum+0; i++)
	    printf "%" ind "s%s\n", "", packages[i]
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n\n\n\n"
	printf "\n\tTest Engineer:\t\t\t\tSignature:\n"
	printf "\n\tJob Title:\t\t\t\tDate:\n"
	printf "\n\tOrganisation:\n"
	printf "\n\n\n\tClient Name:\t\t\t\tSignature:\n"
	printf "\n\tJob Title:\t\t\t\tDate:\n"
	printf "\n\tOrganisation:\n"
	printf "\n\n\n\tTest Date:\t\t%s\n", vh_date
	printf "\n\tTest System:\t\t%s\n", vsx_sys
	printf ".bp\n\n\n\n\n\n"
	str = "Report Contents"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
		printf "_"
	printf "\n\n\n"
	printf "\n\t1. Operational Summary\n"
	printf "\n\t2. Conformance Summary\n"
	printf "\n\t3. FIP Results Signatures\n"
	printf "\n\t4. Detailed Results by Test Section\n"
	printf "\n\t5. Test Parameters\n"
	printf ".bp\n\n\n\n\n"
	str = "Operational Summary"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
		printf "_"
	printf "\n\n\n"
	printf "\n\tTest Date:    \t\t%s\n", vh_date
	printf "\n\tTest Agency:  \t\t%s\n", vsx_org
	printf "\n\tTest Operator:\t\t%s\n\n\n\n", cover["vo"]
	printf "\n\tReport Date:  \t\t%s\n", DTE
	printf "\n\tReport Level: \t\t%s\n", LEVEL
	printf "\n\tReport Range: \t\t%s\n\n\n\n", RANGE
	if (jnlfilenum == 1)
		printf "\n\tJournal File: \t\t%s\n", jnlfiles[1]
	else
	{
		printf "\n\tJournal Files:\n\n"
		for (i = 1; i <= jnlfilenum; i++)
			printf "\t\t%s\n", jnlfiles[i]
	}
	printf "\n\n\n\tSystem Configuration\n\n"
	printf "\t\tHardware Description:\n\n\n\n\n"
	printf "\t\tSoftware Description and Release:\n"
	printf ".bp\n"

	if (TEST_MODE == "POSIX90")
		spec = "POSIX.1-1990"
	else if (TEST_MODE == "POSIX96")
		spec = "POSIX.1-1996"
	else if (TEST_MODE == "FIPS")
		spec = "FIPS 151"
	else if (TEST_MODE == "XPG3")
		spec = "XPG issue 3"
	else if (TEST_MODE == "XPG4")
		spec = "XSH4 version 1"
	else if (TEST_MODE == "UNIX95")
		spec = "XSH4 version 2"
	else if (TEST_MODE == "UNIX98")
		spec = "XSH5 version 1"
	else
		spec = "(unknown test mode " TEST_MODE " )"

	printf "CONFORMANCE Summary\n"
	printf "\n\tThis is to certify that this system when tested for conformance\n"
	if (XNFS == "y")
		printf "\tto XNFS using %s interfaces achieved the results below.\n\n", spec
	else
		printf "\tto %s achieved the results below.\n\n", spec
	if (incomplete == "YES" || vtermok == "NO")
	{
		printf "*****  THESE RESULTS ARE INCOMPLETE DUE TO PREMATURE END OF JOURNAL FILE  *****\n\n"
	}
	printf "\n%s\n", \
"            |    TOTALS   Succeeded   Warnings Unresolved Unsupported NotInUse|"
	printf "%s\n", \
"Section     | Expect Actual       Failed      FIP    Uninitiated  Untested    |"
	printf "%s\n", \
"____________|_________________________________________________________________|"
	printf "%s\n", \
"            |                                                                 |"
	tot[EXP] = tot["exe"] = tot["suc"] = tot["1"] = \
		tot["101"] = tot["102"] = tot["2"] = tot["6"] = \
		tot["4"] = tot["5"] = tot["3"] = 0
	for (sect = 1; sect <= sectnum+0; sect++)
	{
	    for (i = 1; i <= 2; i++)
	    {
		if (i == 1)
			areatype = "F"
		else
			areatype = "M"

		sectid = sections[sect]
		if (count[sectid "," areatype "," EXP] == 0 && \
		    count[sectid "," areatype ",exe"] == 0)
			continue
		if (XNFS == "n" && sectid ~ /\.os$/)
			name = sprintf("%-10s%s", sectid, areatype)
		else
			name = sectid

		printf "%-12s|%6d %6d %6d %5d %4d %4d %5d %5d %5d %4d %4d |\n", \
			name, \
			count[sectid "," areatype "," EXP], \
			count[sectid "," areatype ",exe"], \
			count[sectid "," areatype ",suc"], \
			count[sectid "," areatype ",1"], \
			count[sectid "," areatype ",101"], \
			count[sectid "," areatype ",102"], \
			count[sectid "," areatype ",2"], \
			count[sectid "," areatype ",6"], \
			count[sectid "," areatype ",4"], \
			count[sectid "," areatype ",5"], \
			count[sectid "," areatype ",3"]
		for (ix in tot)
			tot[ix] += count[sectid "," areatype "," ix]
	    }
	}
	printf "%-12s|%6d %6d %6d %5d %4d %4d %5d %5d %5d %4d %4d |\n", \
		"TOTAL", tot[EXP], tot["exe"], tot["suc"], \
		tot["1"], tot["101"], tot["102"], tot["2"], \
		tot["6"], tot["4"], tot["5"], tot["3"]
	printf "%s\n\n", \
"____________|_________________________________________________________________|"

	printf "\n\tNumber of amendments ____________\n"
	str = "Signature/Date"
	ind = (COLS - length(str)) - 6
	printf "%" ind "s", ""
	for (i = 1; i <= length(str) + 5; i++)
	    printf "_"
	printf "\n%" ind "s%s\n", "", str
	printf ".bp\n\n\n"

	str = "FIP Results Signatures"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n"
	if (fip_cnt == 0)
	    printf "\n\n\tNo FIP results obtained.\n"
	for (nfip = 1; nfip <= fip_cnt+0; nfip++)
	{
	    printf "\n\n\t%s\n\n", fip_id[nfip]
	    print "\t\tTest Description:"
	    print fip_td[nfip]
	    print "\t\tTest Information:"
	    print fip_ti[nfip]
	    print "\tSignature/Date ____________________________________"
	}
	printf ".bp\n"
}
