'''	SCCS:	@(#) USER MANUAL dbug.m 4.1 (01/21/97)  VSXgen release 1.4
.TH dbug vlib "UniSoft Ltd." "VSXgen release 1.4" "VSX User Manual"
.SH NAME
DBUG_ENTER, DBUG_LENTER, DBUG_RETURN, DBUG_VOID_RETURN, DBUG_EXIT,
DBUG_PROCESS, DBUG_PUSH, DBUG_POP, DBUG_EXECUTE, DBUG_PRINTF,
DBUG_SETJMP, DBUG_LONGJMP \- debugging macros
.SH SYNOPSIS
.B #include <dbug.h>
.PP
.B DBUG_ENTER(function_name)
.br
.B char *function_name
.PP
.B DBUG_LENTER(function_name, level)
.br
.B char *function_name
.br
.B int level
.PP
.B DBUG_RETURN(value)
.PP
.B DBUG_VOID_RETURN
.PP
.B DBUG_EXIT(exitcode)
.PP
.B DBUG_PROCESS(process_name)
.br
.B char *process_name
.PP
.B DBUG_PUSH(dbug_state)
.br
.B char *dbug_state
.PP
.B DBUG_POP()
.PP
.B DBUG_EXECUTE(keyword, code_block)
.br
.B char *keyword
.PP
.B DBUG_PRINTF(keyword, format
[, arg.... ]
.B )
.br
.B char *keyword
.br
.B char *format
.PP
.B DBUG_SETJMP(env)
.br
.B sigjmp_buf env
.PP
.B DBUG_LONGJMP(env, val)
.br
.B sigjmp_buf env
.br
.B int val
.SH DESCRIPTION
This manual entry summarizes the usage of a set of debugging macros in the
.I dbug
package. The macro definitions are found in the user include file
.B dbug.h
from the standard include directory.
.TP 5
DBUG_ENTER
Used to tell the runtime support module the name of the function
being entered.
The argument must be of type "pointer to character".
The
DBUG_ENTER
macro must precede all executable lines in the
function just entered, and must come after all local declarations.
Each
DBUG_ENTER
macro must have a matching
DBUG_RETURN
or
DBUG_VOID_RETURN
macro
at the function exit points.
DBUG_ENTER
macros used without a matching
DBUG_RETURN
or
DBUG_VOID_RETURN
macro
will cause warning messages from the
.I dbug
package runtime support module.
.br
EX:\ DBUG_ENTER\ ("main");
.TP 5
DBUG_LENTER
This may be used in exactly the same manner as DBUG_ENTER,
except the parameter
.I level
can be used to specify the debugging output level for the function.
Debug lines will then only be generated from the function when
the \fBl\fP option to DBUG_PUSH has been used to select a sufficiently
high output level.
.br
EX: \ DBUG_LENTER\ ("cppair",2);
.TP 5
DBUG_RETURN
Used at each return point of a function containing a
DBUG_ENTER
macro
at the entry point.
The argument is the value to return.
Functions which return no value (void) should use the
DBUG_VOID_RETURN
macro.
It
is an error to have a
DBUG_RETURN
or
DBUG_VOID_RETURN
macro in a function
which has no matching
DBUG_ENTER
macro, and the compiler will complain
if the macros are actually used (expanded).
.br
EX:\ DBUG_RETURN\ (value);
.br
EX:\ DBUG_VOID_RETURN;
.TP 5
DBUG_EXIT
Used in place of a normal exit() call in functions which have
DBUG_ENTER at the entry point.  The argument is passed to exit().
.br
EX:\ DBUG_EXIT\ (exitcode);
.TP 5
DBUG_PROCESS
Used to name the current process being executed.
A typical argument for this macro is "argv[0]", though
it will be perfectly happy with any other string.
.br
EX:\ DBUG_PROCESS\ (argv[0]);
.TP 5
DBUG_PUSH
Sets a new debugger state by pushing the current
.B dbug
state onto an
internal stack and setting up the new state using the debug control
string passed as the macro argument.
The most common usage is to set the state specified by a debug
control string retrieved from the argument list.
Note that the leading "-#" in a debug control string specified
as a command line argument must
.B not
be passed as part of the macro argument.
The proper usage is to pass a pointer to the first character
.B after
the "-#" string.
.br
EX:\ DBUG_PUSH\ (&(argv[i][2]));
.br
EX:\ DBUG_PUSH\ ("d:t");
.br
EX:\ DBUG_PUSH\ ("");
.TP 5
DBUG_POP
Restores the previous debugger state by popping the state stack.
Attempting to pop more states than pushed will be ignored and no
warning will be given.
The
DBUG_POP
macro has no arguments.
.br
EX:\ DBUG_POP\ ();
.TP 5
DBUG_EXECUTE
The DBUG_EXECUTE macro is used to execute any arbitrary C code.
The first argument is the debug keyword, used to trigger execution
of the code specified as the second argument.
This macro must be used cautiously because, like the
DBUG_PRINTF
macro,
it is automatically selected by default whenever the 'd' flag has
no argument list (I.E., a "-#d:t" control string).
.br
EX:\ DBUG_EXECUTE\ ("abort",\ abort\ ());
.br
EX:\ DBUG_EXECUTE\ ("tprint",\ {printtree\ (tree,\ DBUG_FILE);});
.TP 5
DBUG_PRINTF
Used to do printing via the "vfprintf" library function on the
current debug stream.
The first argument is a debug keyword, the second is a format string,
and the remaining arguments, if any, are the values to be printed.
.br
EX:\ DBUG_PRINTF\ ("eof",\ "end\ of\ file\ found");
.br
EX:\ DBUG_PRINTF\ ("type","type\ is\ %x", type);
.br
EX:\ DBUG_PRINTF\ ("stp", "%lx\ ->\ %s", (long)stp, stp\ ->\ name);
.TP 5
DBUG_SETJMP
Used to save the current environment and signal mask of the calling process
as well as the current state of the debugger to be restored later 
by DBUG_LONGJMP.
.TP 5
DBUG_LONGJMP
Used to restore the environment and signal mask of the calling process
as well as the state of the debugger saved earlier
by DBUG_SETJMP.
.PP
DEBUG CONTROL STRING
.PP
The debug control string is used to set the state of the debugger
via the
.B DBUG_PUSH
macro.
This section summarizes the currently available debugger options
and the flag characters which enable or disable them.
Argument lists enclosed in '[' and ']' are optional.
.TP 5
d[,keywords]
Enable output from macros with specified keywords.
A null list of keywords implies that all keywords are selected.
.TP 5
f[,functions]
Limit debugger actions to the specified list of functions
and the functions which they invoke.
A null list of functions implies that all functions are selected.
.TP 5
F
Mark each debugger output line with the name of the source file
containing the macro causing the output.
.TP 5
l[,N]
Specify level of debugging output required, defaults to 1.
Functions entered with DBUG_LENTER will
only generate debugging output if this level is greater than
or equal to the DBUG_LENTER level argument.
.TP 5
L
Mark each debugger output line with the source file line number of
the macro causing the output.
.TP 5
n
Mark each debugger output line with the current function nesting depth.
.TP 5
o[,file]
Redirect the debugger output stream to the specified file.
The default output file is specified by the environment variable
.BR VSX_DBUG_FILE .
If this is not set or cannot be opened, stderr is used.
A null argument list or file name "-" causes output to be sent to stderr.
.TP 5
p[,processes]
Limit debugger actions to the specified processes.
A null list implies all processes.
This is useful for processes which run child processes.
Note that each debugger output line can be marked with the name of
the current process via the 'P' flag.
The process name must match the argument passed to the
.B DBUG_PROCESS
macro.
.TP 5
P
Mark each debugger output line with the name and PID of the current process.
Most useful when used with a process which runs child processes that
are also being debugged.
Note that the parent process must arrange for the debugger control
string to be passed to the child processes.
.TP 5
r
Used in conjunction with the
.B DBUG_PUSH
macro to reset the current
indentation level back to zero.
Most useful with
.B DBUG_PUSH
macros used to temporarily alter the
debugger state.
.TP 5
t[,N]
Enable function control flow tracing.
The maximum nesting depth is specified by N, and defaults to
200.
.SH AUTHOR
Fred Fish, Unisoft Corp. - Original DBUG Author
.SH RELEASE
VSXgen 1.4
