'''	SCCS:	@(#) USER MANUAL vrpt.m	4.2 (04/18/97)  VSXgen release 1.4
.TH vrpt vprog "UniSoft Ltd." "VSXgen release 1.4" "VSX USER MANUAL"
.SH NAME
vrpt - validation test report generator
.SH USAGE
.B vrpt
.IB [ -l level]
.IB [ -r coverage]
.IB [ -f file]
.RB [ -v ]
.RB [ -H ]
.RB [ -P ]
.RB [ -p ]
.br
.ti +5
.IB [ -L len]
.IB [ -W wid]
.IB [ -t lines]
.RI [ jnlfile ...]
.SH DESCRIPTION
.B Vrpt
generates a report from
journal files specified by the
.I jnlfile
argument.
Reports are generated on the standard output.
.PP
Reports can be generated at one of three
.B levels
- the
.BR section ,
.BR area ,
and
.B testset
levels - and covering a specified
range of sections, areas or testsets within these levels.
.PP
At the
.B section
level, for each section specified by the
.B coverage
parameter, a section report is produced listing section start and end
times, and a section summary listing the number of areas and testsets run
and the number of test/make results in each category.  Test result
categories are: Succeeded, Failed, Warning, FIP (Further Information
Provided), Unresolved, Uninitiated, Unsupported, Untested and Not In Use.
Make result categories are: Succeeded, Failed and Unsupported.
.PP
At the
.B area
level, for each area specified by the
.B coverage
parameter, an area report is produced giving area start and end time,
a table showing the number of results in each category (see list
above) for each testset run, a list of all unsuccessful tests under
result category headings, and an area summary listing the
number of testsets run and the number of results in each category.
A section summary is produced after the area reports in each section.
.PP
At the
.B testset
level, for each testset specified by the
.B coverage
parameter, a report is produced listing testset start and end
time, detailed testset results, and a summary listing
the number of results in each category (see list above).  The detailed
results list each unsuccessful test/make with its result category
and any supplementary information produced by the test/make stage.
In verbose mode (-v flag given) all successful and "Not In Use"
tests/makes are also listed.
Area and section summaries are produced after the testset reports
in each area/section.
.PP
Each section, area, or testset level individual report
starts with the section/area/testset identifier
as passed through from the test/make stage.
.PP
Each validation test report run is normally prefaced
by cover pages giving a contents list, an operational summary and
a conformance summary.
Details are described under the "-P" flag description below.
.PP
Any input that
.I vrpt
does not understand will be ignored, with a warning being issued for
the first of a sequence of lines not understood. Processing will attempt to
continue normally from the first understandable line.
.SH PARAMETERS
.SH "Command Line"
.TP
.BI -l level
generate report at
.IR level .
.I Level
can be one of "\fBsect\fP", "\fBarea\fP", or "\fBtset\fP", for
section, area, and testset levels respectively.
.br
.I level
defaults to
.BR tset
if no -l parameter given.
.TP
.BI -r coverage
report only on the specified range of tcc output within the level
specified above.
Note that the tables in the conformance summary cover page always give
the complete results for the journal files being processed - only the
detailed reports are affected when a reduced coverage is specified.
.PP
Coverage can be specified as follows:
.br
1) as a list in the format "name1 name2 ... name\fIn\fP", where
a report will be generated for each section/area/testset (depending on
report level) whose name appears in the list;
.br
2) as a range, in the format "name1:name2", where reporting will start
with the section/area/testset (depending on report level) named
name1 and will continue until the report for section/area/testset
name2 is completed; note that ordering of sections/areas/testsets
within each journal file depends on the scenario files,
and may differ between runs; or,
.br
3) a combination of the above, in the format "name1:name2 name3 name4
name5:name6", etc - with the combination of the above meanings. 
.sp
It is
.I not
an error if specified names do not actually appear in the journal files.
.sp
If range names are specified to a greater level than the level as given in
the -l option, then the extra levels are ignored.  E.g.
.B "-l sect -r section1/area1"
will be processed the same as
.BR "-l sect -r section1" .
.sp
Vrpt defaults to reporting on every section/area/testset in the journal files
if no -r list is specified.
.TP
.BI -f file
take the coverage specifier list from
.I file ;
this file should contain a list of section/area/testset
identifiers or names one per line.
.sp
The
.B -f
and the
.B -r
parameters can be used together - the two lists are merged, and
the resulting list used, ignoring repetitions.
.TP
.B -v
Verbose mode - list names of successful and "Not In Use"
tests/makes in testset level reports.
Without the -v flag, detailed output at this level is produced only for
unsuccessful tests/makes.
.TP
.B -H
Disable page headers and footers.
.I Vrpt
will normally print page headers and footers whose
placement and size depend on the page size
flags given below.  They contain the report level,
system and agency names, test/make date, and page number.
.sp
Vrpt produces headers and footers by default if no -H flag is given.
.TP
.B -P
Do not print the cover pages normally produced with reports, or
the parameter list at the end of all reports.  Test cover pages
consist of a banner page; a contents page; an operational summary
listing test date, test agency and operator, report date, report
level and coverage, and the journal files reported on; and, a conformance
summary showing tables of the total number of tests in each result
category for each section, over the whole of the journal files
being processed (not just the coverage specified with -r).
.sp
Report date is determined from the host system at the time of vrpt
invocation.
Report level and coverage are as given on the vrpt command line.
All other items are determined from the journal files.
.sp
The -P parameter implies -p, so that progressive report output is generated.
.sp
Vrpt defaults to producing the cover pages and parameter list if the
-P flag is not given.
.TP
.B -p
Disable "Page n of N" page numbering style.
.I Vrpt
will normally report the page numbers in the style "Page 4 of 40". However,
this means that no output is generated all the pages have been prepared. 
.B -p
allows the user to disable this feature, and thus allowing progressive
report output.
.sp
.I Vrpt
will produce footers with the "Page n of N" page numbering, and
generates a progress message every 25 pages on stderr, if the -p flag is not
given.
.TP
.BI -L len
Page length is
.I len
lines - used to place headers and footers properly
on the output pages. Defaults to 66 lines if no
-L flag given.
.TP
.BI -W wid
Page width is
.I wid
columns - used to generate headers and footers
and to wrap long lines in the journal file.
Defaults to 80 columns if no -W flag given.
.TP
.BI -t lines
Truncate test failure information after the specified number of lines.
Some tests can produce hundreds of lines of failure information.  This
option may be used to reduce the size of a full report, with the
complete test information for tests of interest subsequently being
obtained by using the
.B -r
option.  The default is no truncation.
.SH "Environment"
.TP
.B VSXBIN
specifies the directory where vrpt executables and scripts reside.  If
.B VSXBIN
is not set this directory is assumed to be
.IR $HOME/BIN .
.SH RETURNS
.IP 0
Report terminated successfully
.IP 1
Unknown option argument
.IP 2
Unrecoverable error during report generation
.SH DIAGNOSTICS
.TP
"warning: input does not start with control sequence"
- input file was probably not a correctly formed test/make journal file
(i.e. file did not start with the special "start report" sequence);
this is not fatal, but may produce some strange output.
.TP
"warning - line XXX: line ignored"
vrpt ignores lines it does not expect to see at that point, or
that appear
to be malformed. Not fatal, and only produced for the first such
error in each sequence of malformed lines in the input file.
.SH EXAMPLES
.TP
.B "vrpt foo"
.br
Generate a validation report from the journal file named foo,
using default levels and coverage (testset/all), producing page
headers and footers and a parameter list, and put the report onto
standard output.
.TP
.B "vrpt -H -P -larea -r'area1 area3:area7' journal[12]"
.br
Generate a validation report from the vtest journal files "journal1"
and "journal2", at the area level, for the area "area1" and every
area from "area3" to area7" inclusive; don't produce any page
headers or footers or cover pages.
.SH "SEE ALSO"
prpt(vprog)
.br
vrptm(vprog)
.SH AUTHORS
Hamish Reid, UniSoft Ltd.
.br
Stuart Boutell, UniSoft Ltd.
.br
J. A. Nave, UniSoft Ltd.
.SH RELEASE
VSXgen 1.4
