/*
*       SCCS:   @(#)	INC/ALLOW.h		4.1 (01/21/97)  VSXgen release 1.4
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

/***********************************************************************

NAME:		ALLOW.h - include file for namespace pollution header tests

PROJECT:	VSX (X/OPEN Validation Suite)
DESCRIPTION:
		chkallow() searches for its argument in two lists of
		allowed symbols defined before inclusion of this file
		as follows:
			static char *allow[] = { "name1", ..., 0 };
			static char *allow_X[] = { "name1", ..., 0 };

		where allow[] lists the POSIX-only symbols, and
		allow_X[] lists the X/Open extensions (not currently
		used: will be re-enabled in XPG4 mode).

		chkdecl() is identical to chkallow() except for the
		failure message produced.

		chktag() searches both allow[] and tag_allow[] for POSIX
		symbols, and both allow_X[] and tag_X[] for additional
		X/Open symbols.

		If any illegal definitions are found, chkallow(), chkdecl()
		and chktag() increment a counter which must be defined
		before inclusion of this file as follows:
			static int errors = 0;
		
		errallow() reports an error message generated by
		hdrdefs(vtools).
		
REQUIRES PRIOR INCLUSION OF: HEADER.h

AUTHOR:		Geoff Clare, UniSoft Ltd.
DATE CREATED:	23 Oct 1989
MODIFICATIONS:
		Geoff Clare, 15 Dec 1989
			Disable X/Open mode testing.
		
		Geoff Clare, 01 Feb 1990
			Add chkdecl() and chktag().
			Remove POSIX_ALLOW and localallow() facilities.
		
		Geoff Clare, 04 Aug 1994
			Add #undef's of all non-reserved identifiers.

************************************************************************/

/*
 * Undefine all non-reserved identifiers used in the code, in case they
 * have been #defined (in the ".M" file) to detect unprotected additional
 * structure members.
 */
#undef match
#undef s1
#undef s2
#undef check
#undef name
#undef symtype
#undef istag
#undef namep
#undef chkallow
#undef chkdecl
#undef chkparm
#undef chktag
#undef errallow
#undef msg

static int
match(s1, s2)
char *s1, *s2;
{
	while (*s1 != '\0' && *s2 != '\0')
		if (*s1++ != *s2++)
			return 0;

	return (*s1 == *s2);
}

static void
check(name, symtype, istag)
char *name;
char *symtype;
int istag;
{
	char **namep;

#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
	exit(UNSUP);
#endif

	if (istag)
	{
		for (namep = tag_allow; *namep != 0; namep++)
			if (match(*namep, name))
				return;

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3
		for (namep = tag_X; *namep != 0; namep++)
			if (match(*namep, name))
				return;
#endif
	}

	for (namep = allow; *namep != 0; namep++)
		if (match(*namep, name))
			return;

#if defined(_XOPEN_SOURCE) && TEST_XPG > 3
	for (namep = allow_X; *namep != 0; namep++)
		if (match(*namep, name))
			return;
#endif

	PRINT("Illegal %s: %s\n", symtype, name);
	errors++;
}

static void
chkallow(name)
char *name;
{
	check(name, "#defined symbol", 0);
}

static void
chkdecl(name)
char *name;
{
	check(name, "declared or 'typedef'ed symbol", 0);
}

static void
chkparm(name)
char *name;
{
	check(name, "parameter name in prototype", 0);
}

static void
chktag(name)
char *name;
{
	check(name, "struct/union/enum tag", 1);
}

static void
errallow(msg)
char *msg;
{
#if defined(_XOPEN_SOURCE) && !(TEST_XPG > 3)
	exit(UNSUP);
#endif
	PRINT("%s\n", msg);
	exit(FAIL);
}
