# !/bin/sh
#
# i18nlib/i18nfuncs.sh
#
# Copyright (c) 1991, 1992, 1997, 1998, 1999, 2001 X/Open Company Ltd.
# (X/Open)
# trading as The Open Group
# Copyright (c) 2001, 2003 International Business Machines Corp.
# All rights reserved except as granted by this License.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the "Artistic License" which comes with this
# Kit, with the following modification:
# a) "executable(s)" should be interpreted to include
# "test case(s)"
# b) if you wish to make changes as defined in clause 2 and 3, and
# distribute a modified version of this package, then
# clauses 3c and 4c are required
# c) Clause 7 is rephrased as follows: "Subroutines supplied by you
# and linked into this Package shall not be considered part of this
# Package".
#
#
# X/OPEN, TRADING AS THE OPEN GROUP, AND IBM CORP. DISCLAIM ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
# LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# You should have received a copy of the Artistic License with this
# Kit, in the file named "Artistic".  If not, we'll be glad to provide one.


i18n_test1()
{
	echo "<<< This is a test function for i18n test driver.>>>"
}


# Check data types are defined in a header file.
#  Usage: 
#   1. header_name=[header file name]
#   2. check_datatypes [datatype1] [datatype2] ...

check_datatypes()
{
  globok=0

  for datatype_name in ${@} ; do
    echo "#include <${header_name}>"	 > ./${datatype_name}.c
    echo "int main(void)"		>> ./${datatype_name}.c
    echo "{"				>> ./${datatype_name}.c
    echo "  ${datatype_name} val;"	>> ./${datatype_name}.c
    echo ""				>> ./${datatype_name}.c
    echo "  return 0;"			>> ./${datatype_name}.c
    echo "}"				>> ./${datatype_name}.c

    cc -o ./${datatype_name}.out ./${datatype_name}.c > /dev/null 2>&1

    if test ${?} = 1 ; then
      tet_infoline " ${datatype_name} is not defined."
      globok=1
    fi
  done

  if test ${globok} = 0 ; then
    return 0
  else
    return 1
  fi
}


# Check macros are defined in a header file.
#  Usage: 
#   1. header_name=[header file name]
#   2. check_macros [macro1] [macro2] ...

check_macros()
{
  globok=0

  for macro_name in ${@} ; do
    echo "#include <${header_name}>"	 > ./${macro_name}.c
    echo "int main(void)"		>> ./${macro_name}.c
    echo "{"				>> ./${macro_name}.c
    echo "#ifdef ${macro_name}"		>> ./${macro_name}.c
    echo "  return 0;"			>> ./${macro_name}.c
    echo "#else"			>> ./${macro_name}.c
    echo "  return 1;"			>> ./${macro_name}.c
    echo "#endif"			>> ./${macro_name}.c
    echo "}"				>> ./${macro_name}.c

    cc -o ./${macro_name}.out ./${macro_name}.c > /dev/null 2>&1
    ./${macro_name}.out

    if test ${?} = 1 ; then
      tet_infoline " ${macro_name} is not defined."
      globok=1
    fi
  done

  if test ${globok} = 0 ; then
    return 0
  else
    return 1
  fi
}


# Check functions are defined in a header file.
#  Usage: 
#   1. header_name=[header file name]
#   2. check_functions [function1] [function2] ...

check_functions()
{
  globok=0

  for function_name in ${@} ; do
    echo "#include <${header_name}>"		 > ./${function_name}.c
    echo ""					>> ./${function_name}.c
    echo "void ${function_name}(void);"		>> ./${function_name}.c
    echo ""					>> ./${function_name}.c
    echo "int main(void)"			>> ./${function_name}.c
    echo "{"					>> ./${function_name}.c
    echo "  return 0;"				>> ./${function_name}.c
    echo "}"					>> ./${function_name}.c

    cc -o ./${function_name}.out ./${function_name}.c > /dev/null 2>&1

    if test ${?} = 0 ; then
      tet_infoline " ${function_name} is not defined."
      globok=1
    fi
  done

  if test ${globok} = 0 ; then
    return 0
  else
    return 1
  fi
}

# Check exit code ($1) and write infomation when FAILed.
# Usage: result_output_code exitcode outputstring
#

result_output_code ()
{
  if [ $1 = 0 ];then
    tet_result PASS
  else
    tet_infoline "$2"
    tet_result FAIL
  fi
}

# Compare two version strings which includes dots (.) as separator.
# $1 -- target
# $2 -- base
# return "GT, "EQ", "LT"
compare_version ()
{
  target=${1}.
  base=${2}.

  LC_ALL_OLD=$LC_ALL
  LC_ALL=C
# result
# EQ  - equal
# LT  - 1st arg (target) is less then 2nd arg (base)
# GT  - 1st arg (target) is greater than 2nd arg (base)
  result="EQ"

  while [ "x$target" != "x" ] || [ "x$base" != "x" ]
  do
    tnum=${target%%.*}
    tnum=${tnum:=-1}
    bnum=${base%%.*}
    bnum=${bnum:=-1}
    target=${target#*.}
    base=${base#*.}

#   echo "tnum = $tnum, bnum = $bnum"
   if [[ $tnum > $bnum ]]; then
      result="GT"
#      echo "$tnum > $bnum"
      break
   elif [[ $tnum < $bnum ]]; then
      result="LT"
#      echo "$tnum < $bnum"
      break
   fi
  done

  LC_ALL=$LC_ALL_OLD
  echo "$result"
}
