###########################################################################
# usersgroups.sh : LSB usersgroups test suite specific common shell functions
# (C) Copyright 2001 The Free Standards Group Inc
# ./SRC/common/lsblib/usersgroups_funcs.sh
#
# Revision 1.1 2001/06/15 15:32:23 Apham
#		Add more functions for testing user* and group* commands.
#
# Revision 1.2 2001/07/19 15:20:04 Apham & Ahayden
#		Adds more descriptive comments.
#
#Current contents
#   
#	usersgroups functions:
#
#	usersgroups_exist()
#	usersgroups_names_exist()
#	user_exist_add()
#
#	group functions:
#	group_dne()
#	gid_dne()
#	execute_groupadd()
#	execute_groupadd_gid()
#	execute_groupadd_override()
#	test_groupadd()
#	test_groupadd_gid()
#	test_groupadd_override()
#	remove_group()
#	unique_group()
#	unique_gid()
#
#	user functions:
#	user_dne()
#	uid_dne()
#	unique_name()
#	unique_id()
#	check_update_db()
#	execute_useradd()	
#	execute_useradd_home()	
#	execute_useradd_comment()	
#	execute_useradd_gid()	
#	execute_useradd_shell()	
#	execute_useradd_uid()	
#	execute_useradd_expire()	
#	execute_useradd_inactive()	
#	execute_useradd_passwd()	
#	test_useradd()
#	test_useradd_home()
#	test_useradd_comment()
#	test_useradd_gid()
#	test_useradd_shell()
#	test_useradd_uid()
#	test_useradd_expire()
#	test_useradd_inactive()
#	test_useradd_passwd()
#	add_test_user_tp29_34()
#	
# These functions return the following codes unless otherwise specified
#
# 	PASS=0
# 	FAIL=1
# 	WARNING=2
# 	UNRESOLVED=3
# 	UNTESTED=4
#
# Note:  The function uberpass is a C function that returns values from the 
#	 password, groups, and shadow files using the standard C API. 
#
############################################################################


###################################################################################
# usersgroups_names_exist
# test for the existence of a user Name and ensure it is in the correct group
# Argument: User Name to test, The group it shoud be in.
# Returns: 0 PASS  	The User Name exist and is in the correct Group
#	   2 WARNING	The User Name exist; however, it is in the wrong Group.
#          3 UNRESOLVED error
#          4 UNTESTED 	The User Name does not exist on the system.
###################################################################################

usersgroups_exist() {
	func_name=usersgroups_exist
	# validate that we have both arguments
	if [ $# -lt 2 ]; then
	   tet_infoline "$func_name: Missing argument(s)"
	   return 3  
        fi
        user_name=$1
        group_name=$2  

	id -n -u $user_name >/dev/null 2>@1
        if [ $? -eq 0 ]; then  #does the user name exist on this system

            if [ `id -n -g $user_name` = $group_name >/dev/null 2>@1 ]; then 
                funcret=0      # Group is correct for the user name
	    else
                tet_infoline "WARNING: User Name ($user_name) is in the wrong group (`id -n -g $user_name`)." 
		tpresult WARNING 
		funcret=2      # user name is in the wrong group 
	    fi
	else 
	    tet_infoline "User Name ($user_name) does not exist."
	    tpresult UNTESTED
            funcret=4
        fi
	return $funcret
}
###################################################################################
# usersgroups_exist
# test for the existence of a user Name and ensure it is in the correct group
# Argument: User Name to test, The group it shoud be in.
# Returns: 0 PASS , The User Name exist and is in the correct Group
#          1 FAIL, The User Name does not exist on the system or
#                  The User Name exist, However it is in the wrong Group.    
#          3 UNRESOLVED error
###################################################################################

usersgroups_names_exist() {

	func_name=usersgroups_names_exist
	# validate that we have both arguments
	if [ $# -lt 2 ]; then
	   tet_infoline "$func_name: Missing argument(s)"
	   return 3  
        fi
        user_name=$1
        group_name=$2  

	id -n -u $user_name >/dev/null 2>@1
        if [ $? -eq 0 ]; then  #does the user name exist on this system

            if [ `id -n -g $user_name` = $group_name >/dev/null 2>@1 ]; then 
                funcret=0      # Group is correct for the user name
	    else
                tet_infoline "User Name ($user_name) is in the wrong group (`id -n -g $user_name`)."  
		funcret=1      # user name is in the wrong group 
	    fi
	else 
            if [ "$user_name" = "root" -o "$user_name" = "bin" -o "$user_name" = "daemon" ] ; then
               tet_infoline "User Name ($user_name) does not exist."   
               funcret=1    # root,bin and daemon must exist 
            else 
               funcret=0
            fi
        fi
	return $funcret
}

user_exist_add()
############################################################################
# This function check to see if the given user exists.  If not; add it. 
############################################################################
{
# make sure the user exist; if not add it.
	user_name=$1
	user_dne $user_name
	if [ $? -eq 0 ]; then
		useradd $user_name > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			tet_infoline "Un-able to add a test user $user_name"
			tpresult UNRESOLVED
			return 1
		fi
	fi
	return 0
}
###############################################################################
# Functions below are for group* commands
###############################################################################

###############################################################################
#if a group exists, return 1, else return 0
###############################################################################
group_dne() {
    func_name=group_dne
    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    ./uberpass -G -n $group_name > /dev/null 2>&1
    res=$?
    
    if [ $res -eq 1 ]; then
	return 0
    elif [ $res -eq 0 ]; then 
    	return 1
    else
	return 3
    fi
}
###############################################################################
#if a gid exists, return 1, else return 0
###############################################################################

gid_dne() {
    func_name=gid_dne
    if [ $# -ne 1 ]; then 
    	tet_infoline "$func_name: missing argument"
	return 2
    fi
    gid_numb=$1
    ./uberpass -X $gid_numb > /dev/null 2>&1
    res=$?
    if [ $res -eq 1 ]; then
	return 0
    elif [ $res -eq 0 ]; then 
    	return 1
    else
	return 3
    fi
}
	
unique_group()
############################################################################
# This function returns a group name that does not exist on the system or 
#		/etc/group
############################################################################
{
	prefix=vsxg
	name=$prefix
	start=3
	group_dne $name
	res=$? 
	user_dne $name
	res1=$? 
	while [ $res -eq 1 -o $res1 -eq 1 ]; do
		start=`expr $start + 1`
		name=$prefix$start
		group_dne $name
		res=$? 
		user_dne $name
		res1=$? 
		if [ $res -eq 3 -o $res1 -eq 3 ]; then
			res=3
			res1=3
			name=vsxgx281
		fi
	done
	echo $name
}

unique_gid()
############################################################################
# This function returns a gid that does not exist on the system or 
#		/etc/group
############################################################################
{
	temp_id=8374
	gid_dne $temp_id
	res=$? 
	while [ $res -eq 1 ]; do
		temp_id=`expr $temp_id + 17`
		gid_dne $temp_id
		res=$? 
		if [ $res -eq 3 ]; then
		 	temp_id=1923	
		fi
	done
	echo $temp_id
}
############################################################################
#Function executes groupadd.   
#
#Caveat from the man page:
#   You  may  not  remove  the  primary group of any existing user.
#   You must remove the user before you remove the group.
#Return code 8 indicates attempt to delete the primary group
############################################################################ 
execute_groupadd() {
    func_name=execute_groupadd
    if [ $# -ne 1 ]; then
		tet_infoline "$func_name: missing argument"
		return 3
    fi
    group_name=$1
    groupadd $group_name >/dev/null 2>&1
    res=$?
    
    if [ $res -eq 0 ]; then
	funcret=0	
    elif [ $res -eq 8 ]; then
        tet_infoline "Attempt to delete primary group for user"
	funcret=3
    else
        tet_infoline "$group_name was not added"
	funcret=1
    fi
    return $funcret
}

###########################################################################
#Function executes groupadd with -g option
###########################################################################
execute_groupadd_gid() {
    func_name=execute_groupadd_gid
    if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    group_gid=$2
    groupadd -g $group_gid $group_name >/dev/null 2>&1
    res=$?

    if [ $res -eq 0 ]; then
	funcret=0	
    elif [ $res -eq 8 ]; then
        tet_infoline "Attempt to delete primary group for user"
	funcret=3
    else
        tet_infoline "$group_name was not added"
	funcret=1
    fi

    return $funcret
}
###########################################################################
#Function executes groupadd with -g -o options
###########################################################################

execute_groupadd_override() {
    func_name=execute_groupadd_override
    if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    group_gid=$2
    groupadd -g $group_gid -o $group_name  >/dev/null 2>&1
    res=$?
    if [ $res -eq 0 ]; then
	funcret=0	
    elif [ $res -eq 8 ]; then
        tet_infoline "Attempt to delete primary group for user"
	funcret=3
    else
        tet_infoline "$group_name was not added"
	funcret=1
    fi		
    return $funcret
}
#########################################################################
#Tests results of groupadd
#########################################################################
test_groupadd() {
    func_name=test_groupadd
    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    group_tester=$(./uberpass -G -n $group_name)
    if [ "$group_tester" = "$group_name" ]; then
	funcret=0
    else
	tet_infoline "Add could not be verified for group $group_name" 
	funcret=1
    fi
    return $funcret
}
##########################################################################
#Tests results of groupadd -g
##########################################################################

test_groupadd_gid() {
    func_name=test_group_gid
    if [ $# -ne 2 ]; then 
    	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    group_gid=$2
    gid_tester=$(./uberpass -G -g $group_name)
    if [ "$gid_tester" = "$group_gid" ]; then
	funcret=0	
    else
 	tet_infoline "GID $group_gid could not be verified for group $group_name" 		
	funcret=1
    fi
    return $funcret
}

##########################################################################
#Tests the groupadd with options -g -o      
##########################################################################
test_groupadd_override() {
    func_name=test_group_override
    if [ $# -ne 3 ]; then 
    	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_name=$1
    other_name=$2
    group_gid=$3
    grp_gid1=$(./uberpass -G -g $group_name)
    grp_gid2=$(./uberpass -G -g $other_name)
    if [ "$grp_gid1" = "$group_gid" -a "$grp_gid2" = "$group_gid" ]; then
	return 0
    else
	return 1
    fi
}

remove_group() {
    func_name=remove_group
    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    group_to_rm=$1
    groupdel $group_to_rm >/dev/null 2>&1
    if [ $? -eq 0 ]; then 
	funcret=0
    else
        tet_infoline "Group $group_to_rm failed to remove"
	funcret=1
    fi
    return $funcret
}

###############################################################################
# Functions below are for user* commands 
###############################################################################

unique_name()
############################################################################
# This function returns a user name that does not exist on the system or 
#		/etc/passwd
############################################################################
{
	prefix=vsx5
	name=$prefix
	start=0
	user_dne $name
	res=$? 
	group_dne $name
	res1=$? 
	while [ $res -eq 1 -o $res1 -eq 1 ]; do
		start=`expr $start + 1`
		name=$prefix$start
		user_dne $name
		res=$? 
		group_dne $name
		res1=$? 
		if [ $res -eq 3 -o $res1 -eq 3 ]; then
			res=3
			res1=3
			name=vsxx21
		fi
	done
	echo $name
}

unique_id()
############################################################################
# This function returns an uid  that does not exist on the system or 
#		/etc/passwd
############################################################################
{
	temp_id=2634
	uid_dne $temp_id
	res=$? 
	while [ $res -eq 1 ]; do
		temp_id=`expr $temp_id + 17`
		uid_dne $temp_id
		res=$? 
		if [ $res -eq 3 ]; then
			temp_id=1926
		fi
	done
	echo $temp_id
}
check_update_db()
############################################################################
# This function makes sure a user's finger info structure is updated to 
#		the given piece of info or return after 2 minutes.
############################################################################
{
# check to see if the user finger-info db is updated
	user_in=$1
	phrase_in=$2
	stat1=$(./uberpass -U -c $user_in)
	stat2=$(./uberpass -U -c $user_in)
	counter=0
	while [ "$stat1" = "$stat2" -a $counter -le 20 ]; do
		sleep 6 
		stat2=$(./uberpass -U -c $user_in)
		counter=`expr $counter + 1`

# in case the db is updated instantaneously
		echo $stat2 | grep $phrase_in > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			counter=21
		fi
	done
}
###########################################################################
#If user id doesn't exist, return 0, else return 1
###########################################################################
		
uid_dne() {

    func_name=uid_dne

    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    
    id_in=$1

    ./uberpass -Z $id_in > /dev/null 2>&1
    res=$?
    
    if [ $res -eq 1 ]; then
	return 0
    elif [ $res -eq 0 ]; then 
    	return 1
    else
	return 3
    fi

}
###########################################################################
#If a user doesn't exist, return 0, else return 1
###########################################################################

user_dne() {

    func_name=user_dne

    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    
    user_name=$1

    ./uberpass -U -u $user_name > /dev/null 2>&1
    res=$?
    
    if [ $res -eq 1 ]; then
	return 0
    elif [ $res -eq 0 ]; then 
    	return 1
    else
	return 3
    fi
    
}
#############################################################################
#Add a user
#############################################################################
execute_useradd() {

    func_name=execute_useradd

    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
    
    user_name=$1

    useradd $user_name >/dev/null 2>&1
    if [ $? -eq 0 ]; then
    	funcret=0
    else
        tet_infoline "User $user_name was not added."
	funcret=1
    fi

    return $funcret
}
#############################################################################
#Test useradd with no options
#############################################################################
test_useradd() {

	func_name=test_user_add

    if [ $# -ne 1 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi
  	
	user_name=$1

	user_tester=$(./uberpass -U -x $user_name)

	if [ "$user_tester" = "$user_name" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name"
		funcret=1
	fi

	return $funcret

}
###########################################################################
#Remove a user
###########################################################################

remove_user() {

	func_name=remove_user
    
	if [ $# -ne 1 ]; then
		tet_infoline "$func_name: missing argument"
		return 2
    fi

	user_to_rm=$1

	userdel $user_to_rm > /dev/null 2>&1

	if [ $? -eq 0 ]; then
		funcret=0
	else
		tet_infoline "User $user_to_rm failed to remove"
		funcret=1
	fi

	return $funcret
}
#############################################################################
#Add user with option -d
#############################################################################

execute_useradd_home() {

	func_name=execute_useradd_home

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_home=$2

	useradd -d $user_home $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
#############################################################################
#Test useradd with option -d
#############################################################################

test_useradd_home() {

	func_name=test_useradd_home

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_home=$2

	user_tester=$(./uberpass -U -d $user_name)

	if [ "$user_tester" = "$user_home" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name with -d $user_home"
		funcret=1
	fi

	return $funcret

}	

##############################################################################
#Add user with option -c
##############################################################################
execute_useradd_comment() {

	func_name=execute_useradd_comment
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_comment=$2

	useradd -c $user_comment $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
#############################################################################
#Tests useradd with option -c
#############################################################################

test_useradd_comment() {

	func_name=test_useradd_comment

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_comment=$2

	user_tester=$(./uberpass -U -c $user_name)

	if [ "$user_tester" = "$user_comment" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name with -c $user_comment"
		funcret=1
	fi

	return $funcret

}	
#############################################################################
#Adds user with -g option
#############################################################################
execute_useradd_gid() {

	func_name=execute_useradd_gid
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_gid=$2

	useradd -g $user_gid $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
#############################################################################
#Tests useradd with -g option
#############################################################################
test_useradd_gid() {

	func_name=test_useradd_gid

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	group_name=$2

	user_tester=$(./uberpass -U -n $user_name)

	if [ "$user_tester" = "$group_name" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name with -g $group_name"
		funcret=1
	fi

	return $funcret

}	
#############################################################################
#Add user with -s option
#############################################################################
 
execute_useradd_shell() {

	func_name=execute_useradd_shell
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_shell=$2

	useradd -s $user_shell $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
#############################################################################
#Tests useradd with -s option
#############################################################################

test_useradd_shell() {

	func_name=test_useradd_shell

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_shell=$2

	user_tester=$(./uberpass -U -s $user_name)

	if [ "$user_tester" = "$user_shell" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name with -s $user_shell"
		funcret=1

	fi

	return $funcret

}	
#############################################################################
#Adds user with -u option
#############################################################################
 
execute_useradd_uid() {

	func_name=execute_useradd_uid
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_uid=$2

	useradd -u $user_uid $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
#############################################################################
#Tests useradd with the -u options
#############################################################################

test_useradd_uid() {

	func_name=test_useradd_uid

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_uid=$2

	user_tester=$(./uberpass -U -u $user_name)

	if [ "$user_tester" = "$user_uid" ]; then
		funcret=0
	else
		tet_infoline "Add could not be verified for user $user_name with -u $user_uid"
		funcret=1
	fi
	
	return $funcret
}	
#############################################################################
#Adds user with the -e option
#############################################################################

execute_useradd_expire() {

	func_name=execute_useradd_expire
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_expire=$2

	useradd -e $user_expire $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "Can not useradd -e $use_expire $user_name"
		funcret=1
    fi

	return $funcret
}
##############################################################################
#Test useradd with the -e option
##############################################################################

test_useradd_expire() {

	func_name=test_useradd_expire

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_expire=$2

	user_tester=$(./uberpass -S -e $user_name)

	if [ "$user_tester" -eq 1 ]; then
		funcret=0
	else
		tet_infoline "Expiration date for $user_name could not be verified. $user_expire != $user_tester"
		funcret=1
	fi

	return $funcret

}	
############################################################################
#Adds user with -f option
############################################################################
 
execute_useradd_inactive() {

	func_name=execute_useradd_inactive
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_inactive=$2

	useradd -f $user_inactive $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
############################################################################
#Tests users with the -f option
############################################################################

test_useradd_inactive() {

	func_name=test_useradd_inactive

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_inactive=$2

	user_tester=$(./uberpass -S -f $user_name)

	if [ "$user_tester" = "$user_inactive" ]; then
		funcret=0
	else
		tet_infoline "Inactive date could not be verified for user $user_name with -f $user_inactive"
		funcret=1
	fi

	return $funcret

}	
##############################################################################
#Adds user with the -p option
##############################################################################
     
execute_useradd_passwd() {

	func_name=execute_useradd_passwd
	
	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_passwd=$2

	useradd -p $user_passwd $user_name > /dev/null 2>&1

    if [ $? -eq 0 ]; then
		funcret=0
    else
        tet_infoline "User $user_name was not found"
		funcret=1
    fi

	return $funcret
}
##############################################################################
#Test users with the -p option
##############################################################################

test_useradd_passwd() {

	func_name=test_useradd_passwd

	if [ $# -ne 2 ]; then
	tet_infoline "$func_name: missing argument"
	return 2
    fi

	user_name=$1
	user_passwd=$2

	user_tester=$(./uberpass -S -p $user_name)

	if [ "$user_tester" = "$user_passwd" ]; then
		funcret=0
	else
		tet_infoline "Password  could not be verified for user $user_name with -p $user_passwd"
		funcret=1
	fi

	return $funcret
}



#########################################################################
##	This function will add a test user for test cases 29-34 
#########################################################################
add_test_user_tp29_34()
{
#Add this test user for tp29-tp34 
    OTHER_USER=vsxyy123
#Make sure the user doesn't exist.
    user_dne $OTHER_USER
    if [ $? -eq 0 ]; then
        useradd $OTHER_USER > /dev/null 2>&1
        if [ $? -ne 0 ]; then
   	     tet_infoline "WARNING: Un-able to add a test user $OTHER_USER"
        fi
    else 
	tet_infoline "WARNING: change and will delete an existing user: $OTHER_USER"
    fi
}
