#!/bin/sh
# SCCS:	@(#)install/scripts/filldisc.sh	4.1 (01/21/97)  VSXgen release 1.4
#
# filldisc.sh: set up file system for ENOSPC tests
#
# Geoff Clare, UniSoft Ltd., Feb 1990.
# Revised for use from top level Makefile, Nov 1993.
#
# may require unlimited file size privilege

echo "Filling file system mounted on `pwd`"

if test -f filler
then
	echo "filldisc.sh has already been run for this file system"
	exit 0
fi

echo "If the file system is large, this will take a long time ..."

set -e # exit if any command fails

# Create four files with a reasonable size.  These will be removed after the
# disc has been filled so the file system is still useable for other tests.

PAGE_SIZE=$(expr $(getconf PAGE_SIZE) + 1)
dd if=/dev/zero of=fill.tmp1 bs=$PAGE_SIZE count=1
cp fill.tmp1 fill.tmp2
cp fill.tmp1 fill.tmp3
cp fill.tmp1 fill.tmp4

# Create directory
mkdir fill.dir

# Create file for later
> blocks

# Use up all the inodes.

mkdir inodes
cd inodes

set +e	# disable "exit on error"

i=1
ok=y
while test $ok = y
do
	for a in a b c d e f g h i j k l m n o p q r s t u v w x y z
	do
	    if (
	        exec 2>/dev/null
		for b in a b c d e f g h i j k l m n o p q r s t u v w x y z
		do
		    >$a$b$i || exit 1
		done
		exit 0
	    )
	    then :
	    else
		ok=n
		break
	    fi
	done
	i=`expr $i + 1`
done

# use up all the blocks

cd .. || exit 1
while :
do
	 cat fill.tmp[1234] >>blocks 2>/dev/null || break
done

# The file "filler" is a marker that the device has been initialised
# properly.  To return the device to its full state, the ENOSPC tests
# must copy "filler" to the three files "fill.tmp[123]".  Other tests
# should ensure that "fill.tmp[123]" files do not exist before using
# any mounted device (VSX_MOUNT_DEV and VSX_ROFS).

rm -f fill.tmp[123]
mv fill.tmp4 filler
rmdir fill.dir

echo "
File system should now be not quite full:
"
exit 0
