#!/bin/sh

# Configure a loopback file for a test file system if
# this option hasn't already been presented to the user
#
# This is $Revision: 1.8 $
#
# $Log: loopback_setup.sh,v $
#
# Revision 1.8  2007/11/30 13:30:00  rfolco
# find next free loop device 
#
# Revision 1.7  2001/09/19 08:44:25  ajosey
# fixup backslash c in printf
#
# Revision 1.6  2001/06/01 02:00:16  cyeoh
# Merges for lsb-os test suite
#
# Revision 1.5  2001/01/26 02:17:16  cyeoh
# Adds extra path component needed on some platforms
#
# Revision 1.4  2001/01/23 05:41:53  cyeoh
# Fixes problems with script getting called twice by different test set
# setups
# Fixes problems with setting up loop device a second time (or if it
# was being used previously)
#

. scripts/common/loopback.src

if  test ! -f scripts/locks/loopback_setup
then
  touch scripts/locks/loopback_setup
  if test ! -f "loopback_disk.ext2"
  then
      printf "Note: To use a loopback mounted disk you must have the option\n"
      printf "compiled into your kernel.\n"
      printf "Create loopback disk? [y] "
      read cmd
      if [ "$cmd" = "Y" -o "$cmd" = "y" -o "$cmd" = "" ] ; then
	  dd if=/dev/zero of=loopback_disk.ext2 bs=1k count=2000
	  /sbin/mke2fs -F loopback_disk.ext2
	  /sbin/tune2fs -c 0 -i 0 loopback_disk.ext2
      fi
  fi
  
  if test -f "loopback_disk.ext2"
  then
  # Configure loopback disk 
      printf "Setup loopback disk? [y] "
      read cmd
      if [ "$cmd" = "Y" -o "$cmd" = "y" -o "$cmd" = "" ] ; then
	  freedev=`freeloopdev`
	  if [ $? -eq 101 ]; then
	      # no loop devices available, abort!
              printf "Warning: No free loopback devices available."
              # noted in bug 1842 fix - -a isn't accepted everywhere
              # this is just informational - disable
	      #su root -c "PATH=$PATH:/sbin losetup -a"
	      exit $?	
          fi
	  printf "Enter loopback device to use [$freedev]"
	  read cmd
	  if [ "$cmd" = "" ] ; then
	      cmd=$freedev
	  fi
	  PATH=$PATH:/sbin losetup -d $cmd > /dev/null 2>&1; PATH=$PATH:/sbin losetup $cmd loopback_disk.ext2
      fi
  fi
fi

